from grpc.aio._metadata import Metadata

from ..token import token
from .authorization import Authenticator, Provider

HEADER = "authorization"


class TokenAuthenticator(Authenticator):
    def __init__(self, receiver: token.Receiver) -> None:
        super().__init__()
        self._receiver = receiver

    async def authenticate(
        self,
        metadata: Metadata,
        timeout: float | None = None,
        options: dict[str, str] | None = None,
    ) -> None:
        tok = await self._receiver.fetch(timeout=timeout, options=options)
        metadata.add(HEADER, f"Bearer {tok}")

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        return self._receiver.can_retry(err, options)


class TokenProvider(Provider):
    def __init__(self, token_provider: token.Bearer) -> None:
        super().__init__()
        self._provider = token_provider

    def authenticator(self) -> Authenticator:
        return TokenAuthenticator(self._provider.receiver())



from grpc.aio import Channel as GRPCChannel


class AddressChannel:
    def __init__(self, channel: GRPCChannel, address: str) -> None:
        self.address = address
        self.channel = channel


class ChannelBase(GRPCChannel):
    pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1/network_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.vpc.v1 import network_pb2 as nebius_dot_vpc_dot_v1_dot_network__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#nebius/vpc/v1/network_service.proto\x12\rnebius.vpc.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1bnebius/vpc/v1/network.proto\"+\n\x11GetNetworkRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"Z\n\x17GetNetworkByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"v\n\x13ListNetworksRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"l\n\x14ListNetworksResponse\x12,\n\x05items\x18\x01 \x03(\x0b\x32\x16.nebius.vpc.v1.NetworkR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xef\x01\n\x0eNetworkService\x12?\n\x03Get\x12 .nebius.vpc.v1.GetNetworkRequest\x1a\x16.nebius.vpc.v1.Network\x12K\n\tGetByName\x12&.nebius.vpc.v1.GetNetworkByNameRequest\x1a\x16.nebius.vpc.v1.Network\x12O\n\x04List\x12\".nebius.vpc.v1.ListNetworksRequest\x1a#.nebius.vpc.v1.ListNetworksResponseBZ\n\x14\x61i.nebius.pub.vpc.v1B\x13NetworkServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/vpc/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1.network_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.vpc.v1B\023NetworkServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/vpc/v1'
  _GETNETWORKREQUEST.fields_by_name['id']._options = None
  _GETNETWORKREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETNETWORKBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETNETWORKBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETNETWORKBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETNETWORKBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTNETWORKSREQUEST.fields_by_name['parent_id']._options = None
  _LISTNETWORKSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_GETNETWORKREQUEST']._serialized_start=112
  _globals['_GETNETWORKREQUEST']._serialized_end=155
  _globals['_GETNETWORKBYNAMEREQUEST']._serialized_start=157
  _globals['_GETNETWORKBYNAMEREQUEST']._serialized_end=247
  _globals['_LISTNETWORKSREQUEST']._serialized_start=249
  _globals['_LISTNETWORKSREQUEST']._serialized_end=367
  _globals['_LISTNETWORKSRESPONSE']._serialized_start=369
  _globals['_LISTNETWORKSRESPONSE']._serialized_end=477
  _globals['_NETWORKSERVICE']._serialized_start=480
  _globals['_NETWORKSERVICE']._serialized_end=719
# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod
from datetime import datetime, timezone


class Token:
    def __init__(self, token: str, expiration: datetime | None = None) -> None:
        self._tok = token
        if expiration is not None:
            self._exp = expiration
        else:
            expiration = datetime.now(timezone.utc)

    def __str__(self) -> str:
        return self._tok

    @property
    def token(self) -> str:
        return self._tok

    @property
    def expiration(self) -> datetime:
        return self._exp


class Receiver(ABC):
    _latest: Token | None

    @abstractmethod
    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        raise NotImplementedError("Method not implemented!")

    @property
    def latest(self) -> Token | None:
        return self._latest

    async def fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        tok = await self._fetch(timeout=timeout, options=options)
        self._latest = tok
        return tok

    @abstractmethod
    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        return False


class Bearer(ABC):
    @abstractmethod
    def receiver(self) -> Receiver:
        raise NotImplementedError("Method not implemented!")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/spark/v1alpha1/session.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import common_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_common__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import preset_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_preset__pb2
from nebius.api.nebius.msp.v1alpha1 import cluster_pb2 as nebius_dot_msp_dot_v1alpha1_dot_cluster__pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as nebius_dot_msp_dot_v1alpha1_dot_resource_dot_template__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'nebius/msp/spark/v1alpha1/session.proto\x12\x19nebius.msp.spark.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a&nebius/msp/spark/v1alpha1/common.proto\x1a&nebius/msp/spark/v1alpha1/preset.proto\x1a!nebius/msp/v1alpha1/cluster.proto\x1a+nebius/msp/v1alpha1/resource/template.proto\"\xdd\x01\n\x07Session\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x42\n\x04spec\x18\x02 \x01(\x0b\x32&.nebius.msp.spark.v1alpha1.SessionSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12\x46\n\x06status\x18\x03 \x01(\x0b\x32(.nebius.msp.spark.v1alpha1.SessionStatusB\x04\xbaJ\x01\x05R\x06status\"\xbf\x04\n\x0bSessionSpec\x12%\n\x0b\x64\x65scription\x18\x01 \x01(\tH\x00R\x0b\x64\x65scription\x88\x01\x01\x12M\n\x06\x64river\x18\x02 \x01(\x0b\x32-.nebius.msp.spark.v1alpha1.DriverTemplateSpecB\x06\xbaH\x03\xc8\x01\x01R\x06\x64river\x12S\n\x08\x65xecutor\x18\x03 \x01(\x0b\x32/.nebius.msp.spark.v1alpha1.ExecutorTemplateSpecB\x06\xbaH\x03\xc8\x01\x01R\x08\x65xecutor\x12+\n\rspark_version\x18\x04 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x0csparkVersion\x12\x1b\n\tfile_uris\x18\x65 \x03(\tR\x08\x66ileUris\x12\x19\n\x08jar_uris\x18\x66 \x03(\tR\x07jarUris\x12\x1a\n\x08packages\x18g \x03(\tR\x08packages\x12T\n\nspark_conf\x18h \x03(\x0b\x32\x35.nebius.msp.spark.v1alpha1.SessionSpec.SparkConfEntryR\tsparkConf\x12@\n\x06python\x18\xc9\x01 \x01(\x0b\x32\'.nebius.msp.spark.v1alpha1.PythonConfigR\x06python\x1a<\n\x0eSparkConfEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x0e\n\x0c_description\"\xab\x03\n\rSessionStatus\x12>\n\x05phase\x18\x01 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.PhaseR\x05phase\x12>\n\x05state\x18\x02 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.StateR\x05state\x12\x39\n\x16spark_connect_endpoint\x18\x03 \x01(\tH\x00R\x14sparkConnectEndpoint\x88\x01\x01\x12_\n\x15\x64river_preset_details\x18\x04 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.PresetDetailsR\x13\x64riverPresetDetails\x12\x63\n\x17\x65xecutor_preset_details\x18\x05 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.PresetDetailsR\x15\x65xecutorPresetDetailsB\x19\n\x17_spark_connect_endpointBk\n ai.nebius.pub.msp.spark.v1alpha1B\x0cSessionProtoP\x01Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.spark.v1alpha1.session_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n ai.nebius.pub.msp.spark.v1alpha1B\014SessionProtoP\001Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1'
  _SESSION.fields_by_name['metadata']._options = None
  _SESSION.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _SESSION.fields_by_name['spec']._options = None
  _SESSION.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _SESSION.fields_by_name['status']._options = None
  _SESSION.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _SESSIONSPEC_SPARKCONFENTRY._options = None
  _SESSIONSPEC_SPARKCONFENTRY._serialized_options = b'8\001'
  _SESSIONSPEC.fields_by_name['driver']._options = None
  _SESSIONSPEC.fields_by_name['driver']._serialized_options = b'\272H\003\310\001\001'
  _SESSIONSPEC.fields_by_name['executor']._options = None
  _SESSIONSPEC.fields_by_name['executor']._serialized_options = b'\272H\003\310\001\001'
  _SESSIONSPEC.fields_by_name['spark_version']._options = None
  _SESSIONSPEC.fields_by_name['spark_version']._serialized_options = b'\272H\003\310\001\001'
  _globals['_SESSION']._serialized_start=319
  _globals['_SESSION']._serialized_end=540
  _globals['_SESSIONSPEC']._serialized_start=543
  _globals['_SESSIONSPEC']._serialized_end=1118
  _globals['_SESSIONSPEC_SPARKCONFENTRY']._serialized_start=1042
  _globals['_SESSIONSPEC_SPARKCONFENTRY']._serialized_end=1102
  _globals['_SESSIONSTATUS']._serialized_start=1121
  _globals['_SESSIONSTATUS']._serialized_end=1548
# @@protoc_insertion_point(module_scope)



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class UserAccountExternalId(_message.Message):
    __slots__ = ["federation_user_account_id", "federation_id"]
    FEDERATION_USER_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    FEDERATION_ID_FIELD_NUMBER: _ClassVar[int]
    federation_user_account_id: str
    federation_id: str
    def __init__(self, federation_user_account_id: _Optional[str] = ..., federation_id: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/spark/v1alpha1/job_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import job_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_job__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/msp/spark/v1alpha1/job_service.proto\x12\x19nebius.msp.spark.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x18nebius/annotations.proto\x1a#nebius/msp/spark/v1alpha1/job.proto\"\'\n\rGetJobRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"{\n\x0fListJobsRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12$\n\tpage_size\x18\x02 \x01(\x03\x42\x07\xbaH\x04\"\x02(\x00R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x89\x01\n\x10ListJobsResponse\x12\x34\n\x05items\x18\x01 \x03(\x0b\x32\x1e.nebius.msp.spark.v1alpha1.JobR\x05items\x12+\n\x0fnext_page_token\x18\x02 \x01(\tH\x00R\rnextPageToken\x88\x01\x01\x42\x12\n\x10_next_page_token\"\x9d\x02\n\x10\x43reateJobRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12>\n\x04spec\x18\x02 \x01(\x0b\x32\".nebius.msp.spark.v1alpha1.JobSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec:\x80\x01\xbaH}\x1a{\n\x13\x63reate_job.metadata\x12+\'metadata\' must have \'parent_id\' and \'name\'\x1a\x37has(this.metadata.parent_id) && has(this.metadata.name)\"*\n\x10\x43\x61ncelJobRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id2\xf1\x02\n\nJobService\x12O\n\x03Get\x12(.nebius.msp.spark.v1alpha1.GetJobRequest\x1a\x1e.nebius.msp.spark.v1alpha1.Job\x12_\n\x04List\x12*.nebius.msp.spark.v1alpha1.ListJobsRequest\x1a+.nebius.msp.spark.v1alpha1.ListJobsResponse\x12R\n\x06\x43reate\x12+.nebius.msp.spark.v1alpha1.CreateJobRequest\x1a\x1b.nebius.common.v1.Operation\x12R\n\x06\x43\x61ncel\x12+.nebius.msp.spark.v1alpha1.CancelJobRequest\x1a\x1b.nebius.common.v1.Operation\x1a\t\xbaJ\x06sp.mspBn\n ai.nebius.pub.msp.spark.v1alpha1B\x0fJobServiceProtoP\x01Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.spark.v1alpha1.job_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n ai.nebius.pub.msp.spark.v1alpha1B\017JobServiceProtoP\001Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1'
  _GETJOBREQUEST.fields_by_name['id']._options = None
  _GETJOBREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _LISTJOBSREQUEST.fields_by_name['parent_id']._options = None
  _LISTJOBSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTJOBSREQUEST.fields_by_name['page_size']._options = None
  _LISTJOBSREQUEST.fields_by_name['page_size']._serialized_options = b'\272H\004\"\002(\000'
  _CREATEJOBREQUEST.fields_by_name['metadata']._options = None
  _CREATEJOBREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CREATEJOBREQUEST.fields_by_name['spec']._options = None
  _CREATEJOBREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _CREATEJOBREQUEST._options = None
  _CREATEJOBREQUEST._serialized_options = b'\272H}\032{\n\023create_job.metadata\022+\'metadata\' must have \'parent_id\' and \'name\'\0327has(this.metadata.parent_id) && has(this.metadata.name)'
  _CANCELJOBREQUEST.fields_by_name['id']._options = None
  _CANCELJOBREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _JOBSERVICE._options = None
  _JOBSERVICE._serialized_options = b'\272J\006sp.msp'
  _globals['_GETJOBREQUEST']._serialized_start=233
  _globals['_GETJOBREQUEST']._serialized_end=272
  _globals['_LISTJOBSREQUEST']._serialized_start=274
  _globals['_LISTJOBSREQUEST']._serialized_end=397
  _globals['_LISTJOBSRESPONSE']._serialized_start=400
  _globals['_LISTJOBSRESPONSE']._serialized_end=537
  _globals['_CREATEJOBREQUEST']._serialized_start=540
  _globals['_CREATEJOBREQUEST']._serialized_end=825
  _globals['_CANCELJOBREQUEST']._serialized_start=827
  _globals['_CANCELJOBREQUEST']._serialized_end=869
  _globals['_JOBSERVICE']._serialized_start=872
  _globals['_JOBSERVICE']._serialized_end=1241
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as _template_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DriverTemplateSpec(_message.Message):
    __slots__ = ["disk", "resources"]
    DISK_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    disk: _template_pb2.DiskSpec
    resources: _template_pb2.ResourcesSpec
    def __init__(self, disk: _Optional[_Union[_template_pb2.DiskSpec, _Mapping]] = ..., resources: _Optional[_Union[_template_pb2.ResourcesSpec, _Mapping]] = ...) -> None: ...

class DynamicAllocationSpec(_message.Message):
    __slots__ = ["min", "max"]
    MIN_FIELD_NUMBER: _ClassVar[int]
    MAX_FIELD_NUMBER: _ClassVar[int]
    min: int
    max: int
    def __init__(self, min: _Optional[int] = ..., max: _Optional[int] = ...) -> None: ...

class ExecutorTemplateSpec(_message.Message):
    __slots__ = ["disk", "resources", "hosts", "hosts_dynamic_allocation"]
    DISK_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    HOSTS_FIELD_NUMBER: _ClassVar[int]
    HOSTS_DYNAMIC_ALLOCATION_FIELD_NUMBER: _ClassVar[int]
    disk: _template_pb2.DiskSpec
    resources: _template_pb2.ResourcesSpec
    hosts: _template_pb2.HostSpec
    hosts_dynamic_allocation: DynamicAllocationSpec
    def __init__(self, disk: _Optional[_Union[_template_pb2.DiskSpec, _Mapping]] = ..., resources: _Optional[_Union[_template_pb2.ResourcesSpec, _Mapping]] = ..., hosts: _Optional[_Union[_template_pb2.HostSpec, _Mapping]] = ..., hosts_dynamic_allocation: _Optional[_Union[DynamicAllocationSpec, _Mapping]] = ...) -> None: ...



import re
from keyword import iskeyword, issoftkeyword


def first_non_underscore(s: str) -> str:
    for char in s:
        if char != "_":
            return char
    return ""


def is_reserved_name(s: str) -> bool:
    return iskeyword(s) or issoftkeyword(s)


def _modify_name(
    suggested_name: str, container_name: str, *, lower: bool = True
) -> str:
    # get first container name letter
    first_container_letter = first_non_underscore(container_name)
    if first_container_letter == "":
        first_container_letter = "x"
    if lower:
        first_container_letter = first_container_letter.lower()
    else:
        first_container_letter = first_container_letter.upper()

    # if suggested name is reserved or conflicts with getter, prefix it
    if is_reserved_name(suggested_name):
        return f"{first_container_letter}_{suggested_name}"

    # Check keywords conflict pattern
    # We have to do it regardless of actual conflict because it has to be always
    # deterministic and the name has to stay the same
    conflict_with_reserved_words = rf"^({first_container_letter}+)_(.*)$"
    match1 = re.match(conflict_with_reserved_words, suggested_name)
    if match1:
        prefix, something = match1.groups()
        if is_reserved_name(something):
            # Add another letter to the sequence
            return f"{prefix + first_container_letter}_{something}"

    # Check magic conflict pattern
    # As in previous check, we have to check for both the original and the potentially
    # modified names, because we don't want conflicts
    conflict_with_magic_methods = rf"^_({first_container_letter}*)_(.*)__$"
    match2 = re.match(conflict_with_magic_methods, suggested_name)
    if match2:
        prefix, something = match2.groups()
        # Add another letter to the sequence
        return f"_{prefix + first_container_letter}_{something}__"

    # If no pattern matches, return the original string
    return suggested_name


def _class_name(full_name: str) -> str:
    name_parts = full_name.split(".")
    class_name = name_parts[-1]
    container_name = name_parts[-2] if len(name_parts) > 1 else ""
    return _modify_name(class_name, container_name)


# canonical enum names are already pythonic, we have to only check for conflicts
def enum(full_enum_name: str) -> str:
    return _class_name(full_enum_name)


# canonical message names are already pythonic, we have to only check for conflicts
def message(full_enum_name: str) -> str:
    return _class_name(full_enum_name)


# canonical one-of names are already pythonic, we have to only check for conflicts
def one_of(field_name: str, message_name: str) -> str:
    return _modify_name(field_name, message_name)


def service(full_service_name: str) -> str:
    return _class_name(full_service_name)


# canonical field names are already pythonic, we have to only check for conflicts
def field(field_name: str, message_name: str) -> str:
    return _modify_name(field_name, message_name)


# canonical enum value names are already pythonic, we have to only check for conflicts
def enum_value(value_name: str, enum_name: str) -> str:
    return _modify_name(value_name, enum_name, lower=False)


def pascal_to_snake_case(name: str) -> str:
    """
    Converts a PascalCase string to snake_case with double underscores as separators
    and respects abbreviations. Also ensures no collisions by appending unique suffixes
    if needed.
    """
    has_underscore = name[0] == "_"
    # Step 1: Separate PascalCase components
    name = re.sub(r"(_+)", r"_\1", name)
    name = re.sub(r"(?<=[A-Z])([A-Z][a-z])", r"_\1", name)
    name = re.sub(r"(?<=_)([a-z])", r"_\1", name)
    name = re.sub(r"(?<=[a-z0-9])([A-Z])", r"_\1", name)

    if not has_underscore and name[0] == "_":
        name = name[1:]
    # Step 3: Convert to lowercase
    return name.lower()


# convert all pascal methods to snake case
# TODO: create deterministic version that will convert all the names to some
# variant with no collisions, favoring pascal names.
def method(method_name: str, service_name: str) -> str:
    return _modify_name(pascal_to_snake_case(method_name), service_name)



from logging import getLogger
from typing import TypeVar

from grpc.aio._metadata import Metadata as GRPCMetadata

from nebius.base.metadata import Internal, Metadata

log = getLogger(__name__)

MD = TypeVar("MD", Metadata, GRPCMetadata)


def get_options_from_metadata(metadata: MD | None) -> dict[str, str]:
    ret = dict[str, str]()
    if metadata is None:
        return ret
    if isinstance(metadata, GRPCMetadata):
        selected_options = metadata.get_all(Internal.AUTHORIZATION_OPTION.lower())
    elif isinstance(metadata, Metadata):  # type: ignore[unused-ignore]
        selected_options = metadata[Internal.AUTHORIZATION_OPTION]
    for option in selected_options:
        if isinstance(option, bytes):
            option = option.decode("utf-8")
        if isinstance(option, str):
            k, v = option.split("=", 1)
            ret[k] = v
        else:
            log.error(f"invalid auth option type {type(option)}")
    return ret


def add_options_to_metadata(options: dict[str, str], metadata: MD) -> MD:
    for k, v in options.items():
        if "=" in k:
            raise ValueError(f"option key contains '=' {k}")
        metadata.add(Internal.AUTHORIZATION_OPTION.lower(), k + "=" + v)
    return metadata


def options_to_metadata(options: dict[str, str]) -> Metadata:
    return add_options_to_metadata(options, Metadata())



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

import nebius.base.protos.pb_classes as pb_classes
import nebius.api.nebius.msp.v1alpha1.resource.template_pb2 as template_pb2
import nebius.base.protos.descriptor as descriptor
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import nebius.base.protos.unset as unset
import collections.abc as abc
import builtins as builtins
import nebius.api.nebius.msp.v1alpha1.resource.preset_pb2 as preset_pb2
import nebius.api.nebius.msp.v1alpha1.resource.preset_service_pb2 as preset_service_pb2
import nebius.aio.client as client
import grpc as grpc
import nebius.aio.request as request
import nebius.api.nebius.msp.v1alpha1.resource.template_service_pb2 as template_service_pb2
#@ local imports here @#

# file: nebius/msp/v1alpha1/resource/template.proto
class Template(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.Template
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.Template",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        status: "TemplateStatus|template_pb2.TemplateStatus|None|unset.UnsetType" = unset.Unset,
        spec: "TemplateSpec|template_pb2.TemplateSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(status, unset.UnsetType):
            self.status = status
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "status",
            "spec",
        ]
    
    @builtins.property
    def status(self) -> "TemplateStatus":
        return super()._get_field("status", explicit_presence=False,
        wrap=TemplateStatus,
        )
    @status.setter
    def status(self, value: "TemplateStatus|template_pb2.TemplateStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "TemplateSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=TemplateSpec,
        )
    @spec.setter
    def spec(self, value: "TemplateSpec|template_pb2.TemplateSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "status":"status",
        "spec":"spec",
    }
    
class TemplateStatus(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.TemplateStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.TemplateStatus",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        preset_details: "PresetDetails|template_pb2.PresetDetails|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(preset_details, unset.UnsetType):
            self.preset_details = preset_details
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "preset_details",
        ]
    
    @builtins.property
    def preset_details(self) -> "PresetDetails":
        return super()._get_field("preset_details", explicit_presence=False,
        wrap=PresetDetails,
        )
    @preset_details.setter
    def preset_details(self, value: "PresetDetails|template_pb2.PresetDetails|None") -> None:
        return super()._set_field("preset_details",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "preset_details":"preset_details",
    }
    
class TemplateSpec(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.TemplateSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.TemplateSpec",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        resources: "ResourcesSpec|template_pb2.ResourcesSpec|None|unset.UnsetType" = unset.Unset,
        hosts: "Host|template_pb2.Host|None|unset.UnsetType" = unset.Unset,
        disk: "Disk|template_pb2.Disk|None|unset.UnsetType" = unset.Unset,
        role: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(resources, unset.UnsetType):
            self.resources = resources
        if not isinstance(hosts, unset.UnsetType):
            self.hosts = hosts
        if not isinstance(disk, unset.UnsetType):
            self.disk = disk
        if not isinstance(role, unset.UnsetType):
            self.role = role
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "resources",
            "hosts",
            "disk",
            "role",
        ]
    
    @builtins.property
    def resources(self) -> "ResourcesSpec":
        return super()._get_field("resources", explicit_presence=False,
        wrap=ResourcesSpec,
        )
    @resources.setter
    def resources(self, value: "ResourcesSpec|template_pb2.ResourcesSpec|None") -> None:
        return super()._set_field("resources",value,explicit_presence=False,
        )
    
    @builtins.property
    def hosts(self) -> "Host":
        return super()._get_field("hosts", explicit_presence=False,
        wrap=Host,
        )
    @hosts.setter
    def hosts(self, value: "Host|template_pb2.Host|None") -> None:
        return super()._set_field("hosts",value,explicit_presence=False,
        )
    
    @builtins.property
    def disk(self) -> "Disk":
        return super()._get_field("disk", explicit_presence=False,
        wrap=Disk,
        )
    @disk.setter
    def disk(self, value: "Disk|template_pb2.Disk|None") -> None:
        return super()._set_field("disk",value,explicit_presence=False,
        )
    
    @builtins.property
    def role(self) -> "builtins.str":
        return super()._get_field("role", explicit_presence=False,
        )
    @role.setter
    def role(self, value: "builtins.str|None") -> None:
        return super()._set_field("role",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "resources":"resources",
        "hosts":"hosts",
        "disk":"disk",
        "role":"role",
    }
    
class ResourcesSpec(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.ResourcesSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.ResourcesSpec",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        platform: "builtins.str|None|unset.UnsetType" = unset.Unset,
        preset: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(platform, unset.UnsetType):
            self.platform = platform
        if not isinstance(preset, unset.UnsetType):
            self.preset = preset
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "platform",
            "preset",
        ]
    
    @builtins.property
    def platform(self) -> "builtins.str":
        return super()._get_field("platform", explicit_presence=False,
        )
    @platform.setter
    def platform(self, value: "builtins.str|None") -> None:
        return super()._set_field("platform",value,explicit_presence=False,
        )
    
    @builtins.property
    def preset(self) -> "builtins.str":
        return super()._get_field("preset", explicit_presence=False,
        )
    @preset.setter
    def preset(self, value: "builtins.str|None") -> None:
        return super()._set_field("preset",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "platform":"platform",
        "preset":"preset",
    }
    
class PresetDetails(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.PresetDetails
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.PresetDetails",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cpu_count: "builtins.int|None|unset.UnsetType" = unset.Unset,
        memory_gibibytes: "builtins.int|None|unset.UnsetType" = unset.Unset,
        gpu_count: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cpu_count, unset.UnsetType):
            self.cpu_count = cpu_count
        if not isinstance(memory_gibibytes, unset.UnsetType):
            self.memory_gibibytes = memory_gibibytes
        if not isinstance(gpu_count, unset.UnsetType):
            self.gpu_count = gpu_count
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cpu_count",
            "memory_gibibytes",
            "gpu_count",
        ]
    
    @builtins.property
    def cpu_count(self) -> "builtins.int":
        return super()._get_field("cpu_count", explicit_presence=False,
        )
    @cpu_count.setter
    def cpu_count(self, value: "builtins.int|None") -> None:
        return super()._set_field("cpu_count",value,explicit_presence=False,
        )
    
    @builtins.property
    def memory_gibibytes(self) -> "builtins.int":
        return super()._get_field("memory_gibibytes", explicit_presence=False,
        )
    @memory_gibibytes.setter
    def memory_gibibytes(self, value: "builtins.int|None") -> None:
        return super()._set_field("memory_gibibytes",value,explicit_presence=False,
        )
    
    @builtins.property
    def gpu_count(self) -> "builtins.int":
        return super()._get_field("gpu_count", explicit_presence=False,
        )
    @gpu_count.setter
    def gpu_count(self, value: "builtins.int|None") -> None:
        return super()._set_field("gpu_count",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cpu_count":"cpu_count",
        "memory_gibibytes":"memory_gibibytes",
        "gpu_count":"gpu_count",
    }
    
class Range(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.Range
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.Range",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        min: "builtins.int|None|unset.UnsetType" = unset.Unset,
        max: "builtins.int|None|unset.UnsetType" = unset.Unset,
        step: "builtins.int|None|unset.UnsetType" = unset.Unset,
        value: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(min, unset.UnsetType):
            self.min = min
        if not isinstance(max, unset.UnsetType):
            self.max = max
        if not isinstance(step, unset.UnsetType):
            self.step = step
        if not isinstance(value, unset.UnsetType):
            self.value = value
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "min",
            "max",
            "step",
            "value",
        ]
    
    @builtins.property
    def min(self) -> "builtins.int":
        return super()._get_field("min", explicit_presence=False,
        )
    @min.setter
    def min(self, value: "builtins.int|None") -> None:
        return super()._set_field("min",value,explicit_presence=False,
        )
    
    @builtins.property
    def max(self) -> "builtins.int":
        return super()._get_field("max", explicit_presence=False,
        )
    @max.setter
    def max(self, value: "builtins.int|None") -> None:
        return super()._set_field("max",value,explicit_presence=False,
        )
    
    @builtins.property
    def step(self) -> "builtins.int":
        """
         step 0 is no step validation
        """
        
        return super()._get_field("step", explicit_presence=False,
        )
    @step.setter
    def step(self, value: "builtins.int|None") -> None:
        return super()._set_field("step",value,explicit_presence=False,
        )
    
    @builtins.property
    def value(self) -> "builtins.int":
        return super()._get_field("value", explicit_presence=False,
        )
    @value.setter
    def value(self, value: "builtins.int|None") -> None:
        return super()._set_field("value",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "min":"min",
        "max":"max",
        "step":"step",
        "value":"value",
    }
    
class Disk(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.Disk
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.Disk",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        type: "builtins.str|None|unset.UnsetType" = unset.Unset,
        size_gibibytes: "Range|template_pb2.Range|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(type, unset.UnsetType):
            self.type = type
        if not isinstance(size_gibibytes, unset.UnsetType):
            self.size_gibibytes = size_gibibytes
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "type",
            "size_gibibytes",
        ]
    
    @builtins.property
    def type(self) -> "builtins.str":
        return super()._get_field("type", explicit_presence=False,
        )
    @type.setter
    def type(self, value: "builtins.str|None") -> None:
        return super()._set_field("type",value,explicit_presence=False,
        )
    
    @builtins.property
    def size_gibibytes(self) -> "Range":
        return super()._get_field("size_gibibytes", explicit_presence=False,
        wrap=Range,
        )
    @size_gibibytes.setter
    def size_gibibytes(self, value: "Range|template_pb2.Range|None") -> None:
        return super()._set_field("size_gibibytes",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "type":"type",
        "size_gibibytes":"size_gibibytes",
    }
    
class DiskSpec(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.DiskSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.DiskSpec",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        type: "builtins.str|None|unset.UnsetType" = unset.Unset,
        size_gibibytes: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(type, unset.UnsetType):
            self.type = type
        if not isinstance(size_gibibytes, unset.UnsetType):
            self.size_gibibytes = size_gibibytes
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "type",
            "size_gibibytes",
        ]
    
    @builtins.property
    def type(self) -> "builtins.str":
        return super()._get_field("type", explicit_presence=False,
        )
    @type.setter
    def type(self, value: "builtins.str|None") -> None:
        return super()._set_field("type",value,explicit_presence=False,
        )
    
    @builtins.property
    def size_gibibytes(self) -> "builtins.int":
        return super()._get_field("size_gibibytes", explicit_presence=False,
        )
    @size_gibibytes.setter
    def size_gibibytes(self, value: "builtins.int|None") -> None:
        return super()._set_field("size_gibibytes",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "type":"type",
        "size_gibibytes":"size_gibibytes",
    }
    
class Host(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.Host
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.Host",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        count: "Range|template_pb2.Range|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(count, unset.UnsetType):
            self.count = count
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "count",
        ]
    
    @builtins.property
    def count(self) -> "Range":
        return super()._get_field("count", explicit_presence=False,
        wrap=Range,
        )
    @count.setter
    def count(self, value: "Range|template_pb2.Range|None") -> None:
        return super()._set_field("count",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "count":"count",
    }
    
class HostSpec(pb_classes.Message):
    __PB2_CLASS__ = template_pb2.HostSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.HostSpec",template_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        count: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(count, unset.UnsetType):
            self.count = count
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "count",
        ]
    
    @builtins.property
    def count(self) -> "builtins.int":
        return super()._get_field("count", explicit_presence=False,
        )
    @count.setter
    def count(self, value: "builtins.int|None") -> None:
        return super()._set_field("count",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "count":"count",
    }
    
# file: nebius/msp/v1alpha1/resource/preset.proto
class Preset(pb_classes.Message):
    __PB2_CLASS__ = preset_pb2.Preset
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.Preset",preset_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        spec: "PresetSpec|preset_pb2.PresetSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "spec",
        ]
    
    @builtins.property
    def spec(self) -> "PresetSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=PresetSpec,
        )
    @spec.setter
    def spec(self, value: "PresetSpec|preset_pb2.PresetSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "spec":"spec",
    }
    
class PresetSpec(pb_classes.Message):
    __PB2_CLASS__ = preset_pb2.PresetSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.PresetSpec",preset_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        flavor: "FlavorSpec|preset_pb2.FlavorSpec|None|unset.UnsetType" = unset.Unset,
        hosts: "Host|template_pb2.Host|None|unset.UnsetType" = unset.Unset,
        disk: "Disk|template_pb2.Disk|None|unset.UnsetType" = unset.Unset,
        role: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(flavor, unset.UnsetType):
            self.flavor = flavor
        if not isinstance(hosts, unset.UnsetType):
            self.hosts = hosts
        if not isinstance(disk, unset.UnsetType):
            self.disk = disk
        if not isinstance(role, unset.UnsetType):
            self.role = role
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "flavor",
            "hosts",
            "disk",
            "role",
        ]
    
    @builtins.property
    def flavor(self) -> "FlavorSpec":
        return super()._get_field("flavor", explicit_presence=False,
        wrap=FlavorSpec,
        )
    @flavor.setter
    def flavor(self, value: "FlavorSpec|preset_pb2.FlavorSpec|None") -> None:
        return super()._set_field("flavor",value,explicit_presence=False,
        )
    
    @builtins.property
    def hosts(self) -> "Host":
        return super()._get_field("hosts", explicit_presence=False,
        wrap=Host,
        )
    @hosts.setter
    def hosts(self, value: "Host|template_pb2.Host|None") -> None:
        return super()._set_field("hosts",value,explicit_presence=False,
        )
    
    @builtins.property
    def disk(self) -> "Disk":
        return super()._get_field("disk", explicit_presence=False,
        wrap=Disk,
        )
    @disk.setter
    def disk(self, value: "Disk|template_pb2.Disk|None") -> None:
        return super()._set_field("disk",value,explicit_presence=False,
        )
    
    @builtins.property
    def role(self) -> "builtins.str":
        return super()._get_field("role", explicit_presence=False,
        )
    @role.setter
    def role(self, value: "builtins.str|None") -> None:
        return super()._set_field("role",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "flavor":"flavor",
        "hosts":"hosts",
        "disk":"disk",
        "role":"role",
    }
    
class FlavorSpec(pb_classes.Message):
    __PB2_CLASS__ = preset_pb2.FlavorSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.FlavorSpec",preset_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cpu: "CpuSpec|preset_pb2.CpuSpec|None|unset.UnsetType" = unset.Unset,
        memory: "MemorySpec|preset_pb2.MemorySpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cpu, unset.UnsetType):
            self.cpu = cpu
        if not isinstance(memory, unset.UnsetType):
            self.memory = memory
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cpu",
            "memory",
        ]
    
    @builtins.property
    def cpu(self) -> "CpuSpec":
        return super()._get_field("cpu", explicit_presence=False,
        wrap=CpuSpec,
        )
    @cpu.setter
    def cpu(self, value: "CpuSpec|preset_pb2.CpuSpec|None") -> None:
        return super()._set_field("cpu",value,explicit_presence=False,
        )
    
    @builtins.property
    def memory(self) -> "MemorySpec":
        return super()._get_field("memory", explicit_presence=False,
        wrap=MemorySpec,
        )
    @memory.setter
    def memory(self, value: "MemorySpec|preset_pb2.MemorySpec|None") -> None:
        return super()._set_field("memory",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cpu":"cpu",
        "memory":"memory",
    }
    
class CpuSpec(pb_classes.Message):
    __PB2_CLASS__ = preset_pb2.CpuSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.CpuSpec",preset_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        count: "builtins.int|None|unset.UnsetType" = unset.Unset,
        generation: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(count, unset.UnsetType):
            self.count = count
        if not isinstance(generation, unset.UnsetType):
            self.generation = generation
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "count",
            "generation",
        ]
    
    @builtins.property
    def count(self) -> "builtins.int":
        return super()._get_field("count", explicit_presence=False,
        )
    @count.setter
    def count(self, value: "builtins.int|None") -> None:
        return super()._set_field("count",value,explicit_presence=False,
        )
    
    @builtins.property
    def generation(self) -> "builtins.int":
        return super()._get_field("generation", explicit_presence=False,
        )
    @generation.setter
    def generation(self, value: "builtins.int|None") -> None:
        return super()._set_field("generation",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "count":"count",
        "generation":"generation",
    }
    
class MemorySpec(pb_classes.Message):
    __PB2_CLASS__ = preset_pb2.MemorySpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.MemorySpec",preset_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        limit_gibibytes: "builtins.int|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(limit_gibibytes, unset.UnsetType):
            self.limit_gibibytes = limit_gibibytes
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "limit_gibibytes",
        ]
    
    @builtins.property
    def limit_gibibytes(self) -> "builtins.int":
        return super()._get_field("limit_gibibytes", explicit_presence=False,
        )
    @limit_gibibytes.setter
    def limit_gibibytes(self, value: "builtins.int|None") -> None:
        return super()._set_field("limit_gibibytes",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "limit_gibibytes":"limit_gibibytes",
    }
    
# file: nebius/msp/v1alpha1/resource/preset_service.proto
class ListPresetsRequest(pb_classes.Message):
    __PB2_CLASS__ = preset_service_pb2.ListPresetsRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.ListPresetsRequest",preset_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "page_size",
            "page_token",
            "parent_id",
        ]
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "page_size":"page_size",
        "page_token":"page_token",
        "parent_id":"parent_id",
    }
    
class ListPresetsResponse(pb_classes.Message):
    __PB2_CLASS__ = preset_service_pb2.ListPresetsResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.ListPresetsResponse",preset_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[Preset]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Preset]":
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Preset,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Preset]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    

class PresetServiceClient(client.Client):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.msp.v1alpha1.resource.PresetService",preset_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    __service_name__ = ".nebius.msp.v1alpha1.resource.PresetService"
    
    def list(self,
        request: "ListPresetsRequest",
        metadata: abc.Iterable[builtins.tuple[builtins.str,builtins.str]]|None = None,
        timeout: builtins.float|None = None,
        credentials: grpc.CallCredentials | None = None,
        compression: grpc.Compression | None = None,
        retries: builtins.int | None = 3,
        per_retry_timeout: builtins.float | None = None,
    ) -> request.Request["ListPresetsRequest","ListPresetsResponse"]:
        return super().request(
            method="List",
            request=request,
            result_pb2_class=preset_service_pb2.ListPresetsResponse,
            metadata=metadata,
            timeout=timeout,
            credentials=credentials,
            compression=compression,
            retries=retries,
            per_retry_timeout=per_retry_timeout,
            result_wrapper=pb_classes.simple_wrapper(ListPresetsResponse),
        )
    

# file: nebius/msp/v1alpha1/resource/template_service.proto
class ListTemplatesRequest(pb_classes.Message):
    __PB2_CLASS__ = template_service_pb2.ListTemplatesRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.ListTemplatesRequest",template_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "page_size",
            "page_token",
            "parent_id",
        ]
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "page_size":"page_size",
        "page_token":"page_token",
        "parent_id":"parent_id",
    }
    
class ListTemplatesResponse(pb_classes.Message):
    __PB2_CLASS__ = template_service_pb2.ListTemplatesResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.msp.v1alpha1.resource.ListTemplatesResponse",template_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[Template]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Template]":
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Template,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Template]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    

class TemplateServiceClient(client.Client):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.msp.v1alpha1.resource.TemplateService",template_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    __service_name__ = ".nebius.msp.v1alpha1.resource.TemplateService"
    
    def list(self,
        request: "ListTemplatesRequest",
        metadata: abc.Iterable[builtins.tuple[builtins.str,builtins.str]]|None = None,
        timeout: builtins.float|None = None,
        credentials: grpc.CallCredentials | None = None,
        compression: grpc.Compression | None = None,
        retries: builtins.int | None = 3,
        per_retry_timeout: builtins.float | None = None,
    ) -> request.Request["ListTemplatesRequest","ListTemplatesResponse"]:
        return super().request(
            method="List",
            request=request,
            result_pb2_class=template_service_pb2.ListTemplatesResponse,
            metadata=metadata,
            timeout=timeout,
            credentials=credentials,
            compression=compression,
            retries=retries,
            per_retry_timeout=per_retry_timeout,
            result_wrapper=pb_classes.simple_wrapper(ListTemplatesResponse),
        )
    

__all__ = [
    #@ local import names here @#
    "Template",
    "TemplateStatus",
    "TemplateSpec",
    "ResourcesSpec",
    "PresetDetails",
    "Range",
    "Disk",
    "DiskSpec",
    "Host",
    "HostSpec",
    "Preset",
    "PresetSpec",
    "FlavorSpec",
    "CpuSpec",
    "MemorySpec",
    "ListPresetsRequest",
    "ListPresetsResponse",
    "PresetServiceClient",
    "ListTemplatesRequest",
    "ListTemplatesResponse",
    "TemplateServiceClient",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/spark/v1alpha1/preset.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as nebius_dot_msp_dot_v1alpha1_dot_resource_dot_template__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&nebius/msp/spark/v1alpha1/preset.proto\x12\x19nebius.msp.spark.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a+nebius/msp/v1alpha1/resource/template.proto\"\xa3\x01\n\x12\x44riverTemplateSpec\x12\x42\n\x04\x64isk\x18\x03 \x01(\x0b\x32&.nebius.msp.v1alpha1.resource.DiskSpecB\x06\xbaH\x03\xc8\x01\x01R\x04\x64isk\x12I\n\tresources\x18\x04 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.ResourcesSpecR\tresources\"K\n\x15\x44ynamicAllocationSpec\x12\x18\n\x03min\x18\x01 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\x03min\x12\x18\n\x03max\x18\x02 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\x03max\"\xe8\x02\n\x14\x45xecutorTemplateSpec\x12\x42\n\x04\x64isk\x18\x02 \x01(\x0b\x32&.nebius.msp.v1alpha1.resource.DiskSpecB\x06\xbaH\x03\xc8\x01\x01R\x04\x64isk\x12I\n\tresources\x18\x04 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.ResourcesSpecR\tresources\x12>\n\x05hosts\x18\x65 \x01(\x0b\x32&.nebius.msp.v1alpha1.resource.HostSpecH\x00R\x05hosts\x12l\n\x18hosts_dynamic_allocation\x18\x66 \x01(\x0b\x32\x30.nebius.msp.spark.v1alpha1.DynamicAllocationSpecH\x00R\x16hostsDynamicAllocationB\x13\n\nhosts_spec\x12\x05\xbaH\x02\x08\x01\x42j\n ai.nebius.pub.msp.spark.v1alpha1B\x0bPresetProtoP\x01Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.spark.v1alpha1.preset_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n ai.nebius.pub.msp.spark.v1alpha1B\013PresetProtoP\001Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1'
  _DRIVERTEMPLATESPEC.fields_by_name['disk']._options = None
  _DRIVERTEMPLATESPEC.fields_by_name['disk']._serialized_options = b'\272H\003\310\001\001'
  _DYNAMICALLOCATIONSPEC.fields_by_name['min']._options = None
  _DYNAMICALLOCATIONSPEC.fields_by_name['min']._serialized_options = b'\272H\003\310\001\001'
  _DYNAMICALLOCATIONSPEC.fields_by_name['max']._options = None
  _DYNAMICALLOCATIONSPEC.fields_by_name['max']._serialized_options = b'\272H\003\310\001\001'
  _EXECUTORTEMPLATESPEC.oneofs_by_name['hosts_spec']._options = None
  _EXECUTORTEMPLATESPEC.oneofs_by_name['hosts_spec']._serialized_options = b'\272H\002\010\001'
  _EXECUTORTEMPLATESPEC.fields_by_name['disk']._options = None
  _EXECUTORTEMPLATESPEC.fields_by_name['disk']._serialized_options = b'\272H\003\310\001\001'
  _globals['_DRIVERTEMPLATESPEC']._serialized_start=144
  _globals['_DRIVERTEMPLATESPEC']._serialized_end=307
  _globals['_DYNAMICALLOCATIONSPEC']._serialized_start=309
  _globals['_DYNAMICALLOCATIONSPEC']._serialized_end=384
  _globals['_EXECUTORTEMPLATESPEC']._serialized_start=387
  _globals['_EXECUTORTEMPLATESPEC']._serialized_end=747
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/spark/v1alpha1/session_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import session_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/nebius/msp/spark/v1alpha1/session_service.proto\x12\x19nebius.msp.spark.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x18nebius/annotations.proto\x1a\'nebius/msp/spark/v1alpha1/session.proto\"+\n\x11GetSessionRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"Z\n\x17GetSessionByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\x7f\n\x13ListSessionsRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12$\n\tpage_size\x18\x02 \x01(\x03\x42\x07\xbaH\x04\"\x02(\x00R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x91\x01\n\x14ListSessionsResponse\x12\x38\n\x05items\x18\x01 \x03(\x0b\x32\".nebius.msp.spark.v1alpha1.SessionR\x05items\x12+\n\x0fnext_page_token\x18\x02 \x01(\tH\x00R\rnextPageToken\x88\x01\x01\x42\x12\n\x10_next_page_token\"\xaa\x02\n\x14\x43reateSessionRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x42\n\x04spec\x18\x02 \x01(\x0b\x32&.nebius.msp.spark.v1alpha1.SessionSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec:\x85\x01\xbaH\x81\x01\x1a\x7f\n\x17\x63reate_session.metadata\x12+\'metadata\' must have \'parent_id\' and \'name\'\x1a\x37has(this.metadata.parent_id) && has(this.metadata.name)\".\n\x14\x44\x65leteSessionRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id2\xf2\x03\n\x0eSessionService\x12W\n\x03Get\x12,.nebius.msp.spark.v1alpha1.GetSessionRequest\x1a\".nebius.msp.spark.v1alpha1.Session\x12\x63\n\tGetByName\x12\x32.nebius.msp.spark.v1alpha1.GetSessionByNameRequest\x1a\".nebius.msp.spark.v1alpha1.Session\x12g\n\x04List\x12..nebius.msp.spark.v1alpha1.ListSessionsRequest\x1a/.nebius.msp.spark.v1alpha1.ListSessionsResponse\x12V\n\x06\x43reate\x12/.nebius.msp.spark.v1alpha1.CreateSessionRequest\x1a\x1b.nebius.common.v1.Operation\x12V\n\x06\x44\x65lete\x12/.nebius.msp.spark.v1alpha1.DeleteSessionRequest\x1a\x1b.nebius.common.v1.Operation\x1a\t\xbaJ\x06sp.mspBr\n ai.nebius.pub.msp.spark.v1alpha1B\x13SessionServiceProtoP\x01Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.spark.v1alpha1.session_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n ai.nebius.pub.msp.spark.v1alpha1B\023SessionServiceProtoP\001Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1'
  _GETSESSIONREQUEST.fields_by_name['id']._options = None
  _GETSESSIONREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETSESSIONBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETSESSIONBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETSESSIONBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETSESSIONBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTSESSIONSREQUEST.fields_by_name['parent_id']._options = None
  _LISTSESSIONSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTSESSIONSREQUEST.fields_by_name['page_size']._options = None
  _LISTSESSIONSREQUEST.fields_by_name['page_size']._serialized_options = b'\272H\004\"\002(\000'
  _CREATESESSIONREQUEST.fields_by_name['metadata']._options = None
  _CREATESESSIONREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CREATESESSIONREQUEST.fields_by_name['spec']._options = None
  _CREATESESSIONREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _CREATESESSIONREQUEST._options = None
  _CREATESESSIONREQUEST._serialized_options = b'\272H\201\001\032\177\n\027create_session.metadata\022+\'metadata\' must have \'parent_id\' and \'name\'\0327has(this.metadata.parent_id) && has(this.metadata.name)'
  _DELETESESSIONREQUEST.fields_by_name['id']._options = None
  _DELETESESSIONREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _SESSIONSERVICE._options = None
  _SESSIONSERVICE._serialized_options = b'\272J\006sp.msp'
  _globals['_GETSESSIONREQUEST']._serialized_start=241
  _globals['_GETSESSIONREQUEST']._serialized_end=284
  _globals['_GETSESSIONBYNAMEREQUEST']._serialized_start=286
  _globals['_GETSESSIONBYNAMEREQUEST']._serialized_end=376
  _globals['_LISTSESSIONSREQUEST']._serialized_start=378
  _globals['_LISTSESSIONSREQUEST']._serialized_end=505
  _globals['_LISTSESSIONSRESPONSE']._serialized_start=508
  _globals['_LISTSESSIONSRESPONSE']._serialized_end=653
  _globals['_CREATESESSIONREQUEST']._serialized_start=656
  _globals['_CREATESESSIONREQUEST']._serialized_end=954
  _globals['_DELETESESSIONREQUEST']._serialized_start=956
  _globals['_DELETESESSIONREQUEST']._serialized_end=1002
  _globals['_SESSIONSERVICE']._serialized_start=1005
  _globals['_SESSIONSERVICE']._serialized_end=1503
# @@protoc_insertion_point(module_scope)



from collections.abc import Callable
from logging import getLogger
from typing import TypeVar

from grpc.aio._call import UnaryUnaryCall
from grpc.aio._interceptor import ClientCallDetails, UnaryUnaryClientInterceptor

from nebius.base import metadata

HEADER = "X-Idempotency-Key"

Req = TypeVar("Req")
Res = TypeVar("Res")

log = getLogger(__name__)


class CleaningInterceptor(UnaryUnaryClientInterceptor):  # type: ignore[unused-ignore,misc]
    async def intercept_unary_unary(
        self,
        continuation: Callable[[ClientCallDetails, Req], UnaryUnaryCall | Res],
        client_call_details: ClientCallDetails,
        request: Req,
    ) -> UnaryUnaryCall | Res:
        if client_call_details.metadata is not None:
            to_delete = {
                k
                for k, _ in client_call_details.metadata
                if k.lower().startswith(metadata.Internal.PREFIX.lower())
            }
            for k in to_delete:
                client_call_details.metadata.delete_all(k)
        return await continuation(client_call_details, request)  # type: ignore



from asyncio import (
    FIRST_COMPLETED,
    AbstractEventLoop,
    CancelledError,
    Task,
    create_task,
    gather,
    get_event_loop,
    iscoroutine,
    new_event_loop,
    run_coroutine_threadsafe,
    sleep,
    wait,
)
from collections.abc import Awaitable, Coroutine, Sequence
from inspect import isawaitable
from logging import getLogger
from typing import Any, TypeVar

from google.protobuf.message import Message
from grpc import (
    CallCredentials,
    ChannelConnectivity,
    ChannelCredentials,
    Compression,
    ssl_channel_credentials,
)
from grpc.aio import Channel as GRPCChannel
from grpc.aio._base_call import UnaryUnaryCall
from grpc.aio._base_channel import (
    StreamStreamMultiCallable,
    StreamUnaryMultiCallable,
    UnaryStreamMultiCallable,
    UnaryUnaryMultiCallable,
)
from grpc.aio._channel import (
    insecure_channel,  # type: ignore[unused-ignore]
    secure_channel,  # type: ignore[unused-ignore]
)
from grpc.aio._interceptor import ClientInterceptor
from grpc.aio._typing import (
    ChannelArgumentType,
    DeserializingFunction,
    MetadataType,
    SerializingFunction,
)

from nebius.aio._cleaner import CleaningInterceptor
from nebius.aio.abc import GracefulInterface
from nebius.aio.authorization.authorization import Provider as AuthorizationProvider
from nebius.aio.authorization.interceptor import AuthorizationInterceptor
from nebius.aio.authorization.token import TokenProvider
from nebius.aio.idempotency import IdempotencyKeyInterceptor
from nebius.aio.service_descriptor import ServiceStub, from_stub_class
from nebius.aio.token import exchangeable, renewable
from nebius.aio.token.static import Bearer as StaticTokenBearer
from nebius.aio.token.static import EnvBearer
from nebius.aio.token.token import Bearer as TokenBearer
from nebius.aio.token.token import Token
from nebius.api.nebius.common.v1.operation_service_pb2_grpc import (
    OperationServiceStub,
)
from nebius.api.nebius.common.v1alpha1.operation_service_pb2_grpc import (
    OperationServiceStub as OperationServiceStubDeprecated,
)
from nebius.base.constants import DOMAIN
from nebius.base.error import SDKError
from nebius.base.methods import service_from_method_name
from nebius.base.options import COMPRESSION, INSECURE, pop_option
from nebius.base.resolver import Chain, Conventional, Resolver, TemplateExpander
from nebius.base.service_account.service_account import (
    TokenRequester as ServiceAccountReader,
)
from nebius.base.tls_certificates import get_system_certificates

from .base import AddressChannel, ChannelBase

logger = getLogger(__name__)

Req = TypeVar("Req", bound=Message)
Res = TypeVar("Res", bound=Message)

T = TypeVar("T")


class LoopError(SDKError):
    pass


class ChannelClosedError(SDKError):
    pass


class NebiusUnaryUnaryMultiCallable(UnaryUnaryMultiCallable[Req, Res]):  # type: ignore[unused-ignore,misc]
    def __init__(
        self,
        channel: "Channel",
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> None:
        super().__init__()
        self._channel = channel
        self._method = method
        self._request_serializer = request_serializer
        self._response_deserializer = response_deserializer

    def __call__(
        self,
        request: Req,
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall[Req, Res]:
        ch = self._channel.get_channel_by_method(self._method)
        ret = ch.channel.unary_unary(  # type: ignore[unused-ignore,call-arg,assignment,misc]
            self._method,
            self._request_serializer,
            self._response_deserializer,
        )(  # type: ignore[unused-ignore,call-arg,assignment,misc]
            request,
            timeout=timeout,
            metadata=metadata,  # type: ignore
            credentials=credentials,
            wait_for_ready=wait_for_ready,
            compression=compression,
        )

        def return_channel(cb_arg: Any) -> None:
            nonlocal ch
            logger.debug(f"Done with call {self=}, {cb_arg=}")
            try:
                self._channel.discard_channel(ch)
            except ChannelClosedError:
                self._channel.run_sync(ch.channel.close(None), 0.1)
                # pass
            ch = None  # type: ignore

        ret.add_done_callback(return_channel)
        return ret  # type: ignore[unused-ignore,return-value]


class NoCredentials:
    pass


Credentials = (
    AuthorizationProvider
    | TokenBearer
    | ServiceAccountReader
    | NoCredentials
    | Token
    | str
    | None
)


def _wrap_awaitable(awaitable: Awaitable[T]) -> Coroutine[Any, Any, T]:
    if iscoroutine(awaitable):
        return awaitable
    if not isawaitable(awaitable):
        raise TypeError(
            "An asyncio.Future, a coroutine or an awaitable is "
            + f"required, {type(awaitable)} given"
        )

    async def wrap() -> T:
        return await awaitable

    return wrap()


async def _run_awaitable_with_timeout(
    f: Awaitable[T],
    timeout: float | None = None,
) -> T:
    task = Task(_wrap_awaitable(f), name=f"Task for {f=}")
    tasks: list[Task[Any]] = list[Task[Any]]([task])
    if timeout is not None:
        timer = Task(sleep(timeout), name=f"Timer for {f=}")
        tasks.append(timer)
    done, pending = await wait(
        tasks,
        return_when=FIRST_COMPLETED,
    )
    for p in pending:
        logger.debug(f"Canceling pending task {p}")
        p.cancel()
    await gather(*pending, return_exceptions=True)
    try:
        if task.exception() is not None:
            if task not in done:
                raise TimeoutError("Awaitable timed out") from task.exception()
            raise task.exception()  # type: ignore
    except CancelledError as e:
        if task not in done:
            raise TimeoutError("Awaitable timed out") from e
        raise e
    return task.result()


class Channel(ChannelBase):  # type: ignore[unused-ignore,misc]
    def __init__(
        self,
        *,
        resolver: Resolver | None = None,
        substitutions: dict[str, str] | None = None,
        domain: str = DOMAIN,
        options: ChannelArgumentType | None = None,
        interceptors: Sequence[ClientInterceptor] | None = None,
        address_options: dict[str, ChannelArgumentType] | None = None,
        address_interceptors: dict[str, Sequence[ClientInterceptor]] | None = None,
        credentials: Credentials = None,
        service_account_id: str | None = None,
        service_account_public_key_id: str | None = None,
        service_account_private_key_file_name: str | None = None,
        credentials_file_name: str | None = None,
        tls_credentials: ChannelCredentials | None = None,
        event_loop: AbstractEventLoop | None = None,
        max_free_channels_per_address: int = 2,
    ) -> None:
        import nebius.api.nebius.iam.v1.token_exchange_service_pb2  # type: ignore[unused-ignore] # noqa: F401 - load for registration
        import nebius.api.nebius.iam.v1.token_exchange_service_pb2_grpc  # noqa: F401 - load for registration

        substitutions_full = dict[str, str]()
        substitutions_full["{domain}"] = domain
        if substitutions is not None:
            substitutions_full.update(substitutions)

        self._max_free_channels_per_address = max_free_channels_per_address

        self._gracefuls = set[GracefulInterface]()
        self._tasks = set[Task[Any]]()

        self._resolver: Resolver = Conventional()
        if resolver is not None:
            self._resolver = Chain(resolver, self._resolver)
        self._resolver = TemplateExpander(substitutions_full, self._resolver)
        if tls_credentials is None:
            root_ca = get_system_certificates()
            with open(root_ca, "rb") as f:
                trusted_certs = f.read()
            tls_credentials = ssl_channel_credentials(root_certificates=trusted_certs)
        self._tls_credentials = tls_credentials

        self._free_channels = dict[str, list[GRPCChannel]]()
        self._methods = dict[str, str]()

        if options is None:
            options = []
        if interceptors is None:
            interceptors = []
        self._global_options = options
        self._global_interceptors: list[ClientInterceptor] = [
            IdempotencyKeyInterceptor()
        ]
        self._global_interceptors.extend(interceptors)

        if address_options is None:
            address_options = dict[str, ChannelArgumentType]()
        if address_interceptors is None:
            address_interceptors = dict[str, Sequence[ClientInterceptor]]()
        self._address_options = address_options
        self._address_interceptors = address_interceptors

        self._global_interceptors_inner: list[ClientInterceptor] = []

        if credentials is None:
            if credentials_file_name is not None:
                from nebius.base.service_account.credentials_file import (
                    Reader as CredentialsFileReader,
                )

                credentials = CredentialsFileReader(credentials_file_name)
            elif (
                service_account_id is not None
                and service_account_private_key_file_name is not None
                and service_account_public_key_id is not None
            ):
                from nebius.base.service_account.pk_file import Reader as PKFileReader

                credentials = PKFileReader(
                    service_account_private_key_file_name,
                    service_account_public_key_id,
                    service_account_id,
                )
            else:
                credentials = EnvBearer()
        if isinstance(credentials, str) or isinstance(credentials, Token):
            credentials = StaticTokenBearer(credentials)
        if isinstance(credentials, ServiceAccountReader):
            exchange = exchangeable.Bearer(credentials, self)
            cache = renewable.Bearer(exchange)
            self._gracefuls.add(cache)
            credentials = cache
        if isinstance(credentials, TokenBearer):
            credentials = TokenProvider(credentials)
        if isinstance(credentials, AuthorizationProvider):
            self._global_interceptors_inner.append(
                AuthorizationInterceptor(credentials)
            )
        elif not isinstance(credentials, NoCredentials):  # type: ignore[unused-ignore]
            raise SDKError(f"credentials type is not supported: {type(credentials)}")

        self._event_loop = event_loop
        self._closed = False

        self._global_interceptors_inner.append(CleaningInterceptor())

    def bg_task(self, coro: Awaitable[T]) -> Task[None]:
        """Run a coroutine without awaiting or tracking, and log any exceptions."""

        async def wrapper() -> None:
            try:
                await coro
            except CancelledError:
                pass
            except Exception as e:
                logger.error("Unhandled exception in Channel.bg_task", exc_info=e)

        ret = create_task(wrapper(), name=f"Channel.bg_task for {coro}")
        ret.add_done_callback(lambda x: self._tasks.discard(x))
        self._tasks.add(ret)
        return ret

    def run_sync(self, awaitable: Awaitable[T], timeout: float | None = None) -> T:
        loop_provided = self._event_loop is not None
        if self._event_loop is None:
            try:
                self._event_loop = get_event_loop()
            except RuntimeError:
                self._event_loop = new_event_loop()

        if self._event_loop.is_running():
            if loop_provided:
                try:
                    if get_event_loop() == self._event_loop:
                        raise LoopError(
                            "Provided loop is equal to current thread's "
                            "loop. Either use async/await or provide "
                            "another loop at the SDK initialization."
                        )
                except RuntimeError:
                    pass
                return run_coroutine_threadsafe(
                    _run_awaitable_with_timeout(awaitable, timeout),
                    self._event_loop,
                ).result()
            else:
                raise LoopError(
                    "Synchronous call inside async context. Either use "
                    "async/await or provide a safe and separate loop "
                    "to run at the SDK initialization."
                )

        return self._event_loop.run_until_complete(
            _run_awaitable_with_timeout(awaitable, timeout)
        )

    def sync_close(self, timeout: float | None = None) -> None:
        return self.run_sync(self.close(), timeout)

    async def close(self, grace: float | None = None) -> None:
        self._closed = True
        awaits = list[Coroutine[Any, Any, Any]]()
        for chans in self._free_channels.values():
            for chan in chans:
                awaits.append(chan.close(grace))
        for graceful in self._gracefuls:
            awaits.append(graceful.close(grace))
        for task in self._tasks:
            task.cancel()
        rets = await gather(*awaits, *self._tasks, return_exceptions=True)
        for ret in rets:
            if isinstance(ret, BaseException) and not isinstance(ret, CancelledError):
                logger.error(f"Error while graceful shutdown: {ret}", exc_info=ret)

    def get_corresponding_operation_service(
        self,
        service_stub_class: type[ServiceStub],
    ) -> OperationServiceStub:
        addr = self.get_addr_from_stub(service_stub_class)
        chan = self.get_channel_by_addr(addr)
        return OperationServiceStub(chan)  # type: ignore[no-untyped-call]

    def get_corresponding_operation_service_alpha(
        self,
        service_stub_class: type[ServiceStub],
    ) -> OperationServiceStubDeprecated:
        addr = self.get_addr_from_stub(service_stub_class)
        chan = self.get_channel_by_addr(addr)
        return OperationServiceStubDeprecated(chan)  # type: ignore[no-untyped-call]

    def get_addr_from_stub(self, service_stub_class: type[ServiceStub]) -> str:
        service = from_stub_class(service_stub_class)
        return self.get_addr_from_service_name(service)

    def get_addr_from_service_name(self, service_name: str) -> str:
        if len(service_name) > 1 and service_name[0] == ".":
            service_name = service_name[1:]
        return self._resolver.resolve(service_name)

    def get_addr_by_method(self, method_name: str) -> str:
        if method_name not in self._methods:
            service_name = service_from_method_name(method_name)
            self._methods[method_name] = self.get_addr_from_service_name(service_name)
        return self._methods[method_name]

    def get_channel_by_addr(self, addr: str) -> AddressChannel:
        if self._closed:
            raise ChannelClosedError("Channel closed")
        if addr not in self._free_channels:
            self._free_channels[addr] = []
        chans = self._free_channels[addr]
        while len(chans) > 0:
            chan = chans.pop()
            if chan.get_state() != ChannelConnectivity.SHUTDOWN:
                return AddressChannel(chan, addr)
            self.bg_task(chan.close(None))

        return self.create_address_channel(addr)

    def return_channel(self, chan: AddressChannel | None) -> None:
        if chan is None:
            return
        if self._closed:
            raise ChannelClosedError("Channel closed")
        if chan.address not in self._free_channels:
            self._free_channels[chan.address] = []
        if (
            chan.channel.get_state() != ChannelConnectivity.SHUTDOWN
            and len(self._free_channels[chan.address])
            < self._max_free_channels_per_address
        ):
            self._free_channels[chan.address].append(chan.channel)
        else:
            self.discard_channel(chan)

    def discard_channel(self, chan: AddressChannel | None) -> None:
        if chan is None:
            return
        if self._closed:
            raise ChannelClosedError("Channel closed")
        self.bg_task(chan.channel.close(None))

    def get_channel_by_method(self, method_name: str) -> AddressChannel:
        addr = self.get_addr_by_method(method_name)
        return self.get_channel_by_addr(addr)

    def get_address_options(self, addr: str) -> ChannelArgumentType:
        ret = [opt for opt in self._global_options]
        if addr in self._address_options:
            ret.extend(self._address_options[addr])
        return ret

    def get_address_interceptors(self, addr: str) -> Sequence[ClientInterceptor]:
        ret = [opt for opt in self._global_interceptors]
        if addr in self._address_interceptors:
            ret.extend(self._address_interceptors[addr])
        ret.extend(self._global_interceptors_inner)
        return ret

    def create_address_channel(self, addr: str) -> AddressChannel:
        logger.debug(f"creating channel for {addr=}")
        opts = self.get_address_options(addr)
        opts, insecure = pop_option(opts, INSECURE, bool)
        opts, compression = pop_option(opts, COMPRESSION, Compression)
        interceptors = self.get_address_interceptors(addr)
        if insecure:
            return AddressChannel(
                insecure_channel(addr, opts, compression, interceptors),  # type: ignore[unused-ignore,no-any-return]
                addr,
            )
        else:
            return AddressChannel(
                secure_channel(  # type: ignore[unused-ignore,no-any-return]
                    addr,
                    self._tls_credentials,
                    opts,
                    compression,
                    interceptors,
                ),
                addr,
            )

    def unary_unary(  # type: ignore[unused-ignore,override]
        self,
        method_name: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> UnaryUnaryMultiCallable[Req, Res]:  # type: ignore[unused-ignore,override]
        return NebiusUnaryUnaryMultiCallable(
            self,
            method_name,
            request_serializer,
            response_deserializer,
        )

    async def __aenter__(self) -> "Channel":
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        await self.close(None)

    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity:
        if self._closed:
            return ChannelConnectivity.SHUTDOWN
        return ChannelConnectivity.READY

    async def wait_for_state_change(
        self,
        last_observed_state: ChannelConnectivity,
    ) -> None:
        raise NotImplementedError("this method has no meaning for this channel")

    async def channel_ready(self) -> None:
        return

    def unary_stream(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> UnaryStreamMultiCallable[Req, Res]:  # type: ignore[unused-ignore]
        raise NotImplementedError("Method not implemented")

    def stream_unary(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> StreamUnaryMultiCallable:
        raise NotImplementedError("Method not implemented")

    def stream_stream(  # type: ignore[unused-ignore,override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
    ) -> StreamStreamMultiCallable:
        raise NotImplementedError("Method not implemented")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/spark/v1alpha1/cluster.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.msp.v1alpha1 import cluster_pb2 as nebius_dot_msp_dot_v1alpha1_dot_cluster__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'nebius/msp/spark/v1alpha1/cluster.proto\x12\x19nebius.msp.spark.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\x1a!nebius/msp/v1alpha1/cluster.proto\"\xdd\x01\n\x07\x43luster\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x42\n\x04spec\x18\x02 \x01(\x0b\x32&.nebius.msp.spark.v1alpha1.ClusterSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12\x46\n\x06status\x18\x03 \x01(\x0b\x32(.nebius.msp.spark.v1alpha1.ClusterStatusB\x04\xbaJ\x01\x05R\x06status\"\xc3\x02\n\x0b\x43lusterSpec\x12%\n\x0b\x64\x65scription\x18\x01 \x01(\tH\x00R\x0b\x64\x65scription\x88\x01\x01\x12\x41\n\x06limits\x18\x03 \x01(\x0b\x32!.nebius.msp.spark.v1alpha1.LimitsB\x06\xbaH\x03\xc8\x01\x01R\x06limits\x12O\n\rauthorization\x18\x04 \x01(\x0b\x32#.nebius.msp.spark.v1alpha1.PasswordB\x04\xbaJ\x01\x04R\rauthorization\x12\x38\n\x12service_account_id\x18\x05 \x01(\tB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x10serviceAccountId\x12)\n\nnetwork_id\x18\x06 \x01(\tB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\tnetworkIdB\x0e\n\x0c_descriptionJ\x04\x08\x02\x10\x03\"\xe8\x01\n\rClusterStatus\x12>\n\x05phase\x18\x01 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.PhaseR\x05phase\x12>\n\x05state\x18\x02 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.StateR\x05state\x12;\n\x17history_server_endpoint\x18\x03 \x01(\tH\x00R\x15historyServerEndpoint\x88\x01\x01\x42\x1a\n\x18_history_server_endpoint\"c\n\x06Limits\x12\x1f\n\x03\x63pu\x18\x01 \x01(\x03\x42\r\xbaH\n\xc8\x01\x01\"\x05\x18\x80\n \x00R\x03\x63pu\x12\x38\n\x10memory_gibibytes\x18\x02 \x01(\x03\x42\r\xbaH\n\xc8\x01\x01\"\x05\x18\x80P \x00R\x0fmemoryGibibytes\"\xaa\x06\n\x08Password\x12\x9d\x06\n\x08password\x18\x01 \x01(\tB\x80\x06\xbaH\xf9\x05\xba\x01X\n\x13password.min_length\x12\x30The password must be at least 8 characters long.\x1a\x0fsize(this) >= 8\xba\x01T\n\x13password.max_length\x12+The password must not exceed 64 characters.\x1a\x10size(this) <= 64\xba\x01j\n\x13password.ascii_only\x12\x38The password must contain only visible ASCII characters.\x1a\x19matches(this, \'^[!-~]+$\')\xba\x01j\n\x17password.min_lower_case\x12\x38The password must include at least one lowercase letter.\x1a\x15this.matches(\'[a-z]\')\xba\x01j\n\x17password.min_upper_case\x12\x38The password must include at least one uppercase letter.\x1a\x15this.matches(\'[A-Z]\')\xba\x01[\n\x13password.min_digits\x12-The password must include at least one digit.\x1a\x15this.matches(\'[0-9]\')\xba\x01\x9b\x01\n\x14password.min_special\x12ZThe password must include at least one special character: !@#$%^&*()_+-=[]{}<>;:\'\"\\|,./?`~\x1a\'this.matches(\'[!-/]|[:-@]|[[-`]|[{-~]\')\xc8\x01\x01\xc0J\x01R\x08passwordBk\n ai.nebius.pub.msp.spark.v1alpha1B\x0c\x43lusterProtoP\x01Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.spark.v1alpha1.cluster_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n ai.nebius.pub.msp.spark.v1alpha1B\014ClusterProtoP\001Z7github.com/nebius/gosdk/proto/nebius/msp/spark/v1alpha1'
  _CLUSTER.fields_by_name['metadata']._options = None
  _CLUSTER.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CLUSTER.fields_by_name['spec']._options = None
  _CLUSTER.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _CLUSTER.fields_by_name['status']._options = None
  _CLUSTER.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _CLUSTERSPEC.fields_by_name['limits']._options = None
  _CLUSTERSPEC.fields_by_name['limits']._serialized_options = b'\272H\003\310\001\001'
  _CLUSTERSPEC.fields_by_name['authorization']._options = None
  _CLUSTERSPEC.fields_by_name['authorization']._serialized_options = b'\272J\001\004'
  _CLUSTERSPEC.fields_by_name['service_account_id']._options = None
  _CLUSTERSPEC.fields_by_name['service_account_id']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _CLUSTERSPEC.fields_by_name['network_id']._options = None
  _CLUSTERSPEC.fields_by_name['network_id']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _LIMITS.fields_by_name['cpu']._options = None
  _LIMITS.fields_by_name['cpu']._serialized_options = b'\272H\n\310\001\001\"\005\030\200\n \000'
  _LIMITS.fields_by_name['memory_gibibytes']._options = None
  _LIMITS.fields_by_name['memory_gibibytes']._serialized_options = b'\272H\n\310\001\001\"\005\030\200P \000'
  _PASSWORD.fields_by_name['password']._options = None
  _PASSWORD.fields_by_name['password']._serialized_options = b'\272H\371\005\272\001X\n\023password.min_length\0220The password must be at least 8 characters long.\032\017size(this) >= 8\272\001T\n\023password.max_length\022+The password must not exceed 64 characters.\032\020size(this) <= 64\272\001j\n\023password.ascii_only\0228The password must contain only visible ASCII characters.\032\031matches(this, \'^[!-~]+$\')\272\001j\n\027password.min_lower_case\0228The password must include at least one lowercase letter.\032\025this.matches(\'[a-z]\')\272\001j\n\027password.min_upper_case\0228The password must include at least one uppercase letter.\032\025this.matches(\'[A-Z]\')\272\001[\n\023password.min_digits\022-The password must include at least one digit.\032\025this.matches(\'[0-9]\')\272\001\233\001\n\024password.min_special\022ZThe password must include at least one special character: !@#$%^&*()_+-=[]{}<>;:\'\"\\|,./?`~\032\'this.matches(\'[!-/]|[:-@]|[[-`]|[{-~]\')\310\001\001\300J\001'
  _globals['_CLUSTER']._serialized_start=194
  _globals['_CLUSTER']._serialized_end=415
  _globals['_CLUSTERSPEC']._serialized_start=418
  _globals['_CLUSTERSPEC']._serialized_end=741
  _globals['_CLUSTERSTATUS']._serialized_start=744
  _globals['_CLUSTERSTATUS']._serialized_end=976
  _globals['_LIMITS']._serialized_start=978
  _globals['_LIMITS']._serialized_end=1077
  _globals['_PASSWORD']._serialized_start=1080
  _globals['_PASSWORD']._serialized_end=1890
# @@protoc_insertion_point(module_scope)
